# -*- coding: utf-8 -*-
#######################################################################
#
#    YAMP - Yet Another Music Player
#    Version 2.7.0 beta 7 2016-02-27
#    Coded by JohnHenry (c)2013
#    Extended by AlfredENeumann (c)2016
#    Support: www.vuplus-support.org
#
#    This program is free software; you can redistribute it and/or
#    modify it under the terms of the GNU General Public License
#    as published by the Free Software Foundation; either version 2
#    of the License, or (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#######################################################################

from Components.ActionMap import ActionMap
from Components.config import config, ConfigSubsection, ConfigDirectory, ConfigYesNo, ConfigSelection, ConfigInteger, ConfigText, getConfigListEntry, configfile
from Components.ConfigList import ConfigListScreen
from Components.FileList import FileList
from Components.Label import Label
from Components.Sources.StaticText import StaticText   #!!!!!neu

import os
from enigma import eTimer

from Screens.Screen import Screen

from __init__ import _

from myLogger import LOG
#from Yamp import YampLCDScreenRunning, YampLCDScreen

yampDir = "/usr/lib/enigma2/python/Plugins/Extensions/YampMusicPlayer/"

TEXTLCD = _('Configuration')

class YampConfigScreen(Screen, ConfigListScreen):
	def __init__(self, session):
		Screen.__init__(self, session)
		with open(os.path.join(yampDir, "skins", config.plugins.yampmusicplayer.yampSkin.value, "YampConfig.xml"), 'r') as f:
			self.skin = f.read()
		separator = "".ljust(40,"-")
		self.list = [ ]

		self.list.append(getConfigListEntry(separator + _('     General Adjustments      ') + separator, config.plugins.yampmusicplayer.separator))
		self.list.append(getConfigListEntry(_("YAMP skin (restart required)"), config.plugins.yampmusicplayer.yampSkin))
		self.databasePath = getConfigListEntry(_("YAMP database directory"), config.plugins.yampmusicplayer.databasePath)
		self.list.append(self.databasePath)
		self.filebrowserPath = getConfigListEntry(_("Filebrowser startup directory"), config.plugins.yampmusicplayer.filebrowserPath)
		self.list.append(self.filebrowserPath)
		self.list.append(getConfigListEntry(_("Save filebrowser directory on exit"), config.plugins.yampmusicplayer.saveLastFilebrowserPath))
		self.playlistPath = getConfigListEntry(_("Directory for saved playlists"), config.plugins.yampmusicplayer.playlistPath)
		self.list.append(self.playlistPath)
		self.list.append(getConfigListEntry(_("Use single directory for lyrics"), config.plugins.yampmusicplayer.useSingleLyricsPath))
		self.lyricsPath = getConfigListEntry(_("Directory for saved lyrics"), config.plugins.yampmusicplayer.lyricsPath)
		self.list.append(self.lyricsPath)
		self.screenSaverSlidePath = getConfigListEntry(_("Screensaver slideshow directory"), config.plugins.yampmusicplayer.screenSaverSlidePath)
		self.list.append(self.screenSaverSlidePath)
		self.screenSaverArtworkPath = getConfigListEntry(_("Artwork slideshow base directory"), config.plugins.yampmusicplayer.screenSaverArtworkPath)
		self.list.append(self.screenSaverArtworkPath)

		self.list.append(getConfigListEntry(separator + _('   Databaselist / Playlist    ') + separator, config.plugins.yampmusicplayer.separator))
		self.list.append(getConfigListEntry(_("New layout of database list"), config.plugins.yampmusicplayer.newLayoutDbList))
		self.list.append(getConfigListEntry(_("Title only once in artist/genre selection"), config.plugins.yampmusicplayer.titleOnlyOnceInSelection))
		self.list.append(getConfigListEntry(_("Standardize title and artist names"), config.plugins.yampmusicplayer.capitalizeTitleAndArtist))

		self.list.append(getConfigListEntry(_("Playlist layout"), config.plugins.yampmusicplayer.playlistLayout))
		self.list.append(getConfigListEntry(_("Save playlist on exit"), config.plugins.yampmusicplayer.savePlaylistOnExit))
		self.list.append(getConfigListEntry(_("Repeat playlist at end"), config.plugins.yampmusicplayer.repeatPlaylistAtEnd))
		self.list.append(getConfigListEntry(_("Enable wrap around on move titles in playlist"), config.plugins.yampmusicplayer.wrapMovePlaylist))
		self.list.append(getConfigListEntry(_("Start playing immediate (if playlist exists)"), config.plugins.yampmusicplayer.startImmediate))
		self.list.append(getConfigListEntry(_("Gap correction (1/100 seconds, 0 = off)"),config.plugins.yampmusicplayer.gapCorrection))

		self.list.append(getConfigListEntry(separator + _('         Cover Search         ') + separator, config.plugins.yampmusicplayer.separator))
		self.list.append(getConfigListEntry(_("Cover search priority 1 (highest)"),config.plugins.yampmusicplayer.prioCover1))
		self.list.append(getConfigListEntry(_("Cover search priority 2"),config.plugins.yampmusicplayer.prioCover2))
		self.list.append(getConfigListEntry(_("Cover search priority 3"),config.plugins.yampmusicplayer.prioCover3))
		self.list.append(getConfigListEntry(_("Cover search priority 4 (lowest)"),config.plugins.yampmusicplayer.prioCover4))

		self.list.append(getConfigListEntry(separator + _('        Lyrics Search         ') + separator, config.plugins.yampmusicplayer.separator))
		self.list.append(getConfigListEntry(_("Lyrics search priority 1 (highest)"),config.plugins.yampmusicplayer.prioLyrics1))
		self.list.append(getConfigListEntry(_("Lyrics search priority 2"),config.plugins.yampmusicplayer.prioLyrics2))
		self.list.append(getConfigListEntry(_("Lyrics search priority 3 (lowest)"),config.plugins.yampmusicplayer.prioLyrics3))
		
		self.list.append(getConfigListEntry(separator + _('         Screensaver          ') + separator, config.plugins.yampmusicplayer.separator))
		self.list.append(getConfigListEntry(_("Type of screensaver"), config.plugins.yampmusicplayer.screenSaverMode))
		self.list.append(getConfigListEntry(_("Screensaver pictures in background"),config.plugins.yampmusicplayer.screenSaverBg))
		self.list.append(getConfigListEntry(_("Include subdirs in slideshow (custom or artwork) directory"), config.plugins.yampmusicplayer.screenSaverSubDirs))
		self.list.append(getConfigListEntry(_("Wait for screensaver (seconds, 0 = off)"), config.plugins.yampmusicplayer.screenSaverWait))
		self.list.append(getConfigListEntry(_("Wait for next screensaver slide (seconds)"), config.plugins.yampmusicplayer.screenSaverNextSlide))
		self.list.append(getConfigListEntry(_("Sorting of screensaver pictures"), config.plugins.yampmusicplayer.screenSaverSort))
		self.list.append(getConfigListEntry(_("Screensaver in dblist/filelist"), config.plugins.yampmusicplayer.screenSaverLeft))
		self.list.append(getConfigListEntry(_("Manual activate/deactivate Screensaver by Video Button"), config.plugins.yampmusicplayer.manKeyScreensaver))
		self.list.append(getConfigListEntry(_("Max. Screensaver Off-Time manual (seconds, 0=unlimited)"),config.plugins.yampmusicplayer.maxTimeSSoffMan))
		self.list.append(getConfigListEntry(_("Show background in titles text (visibility)"),config.plugins.yampmusicplayer.showTextBg))
#		self.list.append(getConfigListEntry(_("Show clock while screensaver active"),config.plugins.yampmusicplayer.showClockSS))

		self.list.append(getConfigListEntry(separator + _('          Operation           ') + separator, config.plugins.yampmusicplayer.separator))
		self.list.append(getConfigListEntry(_("Type of control keys"), config.plugins.yampmusicplayer.yampRemote))
		self.list.append(getConfigListEntry(_("Exit button jumps to upper directory in filelist/databaselist"), config.plugins.yampmusicplayer.newExitKey))
		self.list.append(getConfigListEntry(_("Common button for play / pause"), config.plugins.yampmusicplayer.commonPlayPause))
		self.list.append(getConfigListEntry(_("Confirm exit"), config.plugins.yampmusicplayer.yampConfirmExit))

		self.list.append(getConfigListEntry(separator + _('   Lyrics Sroll / Time Edit   ') + separator, config.plugins.yampmusicplayer.separator))
		self.list.append(getConfigListEntry(_("Min. number of lines for scrolling text"),config.plugins.yampmusicplayer.lyricsMinLinesScroll))
		self.list.append(getConfigListEntry(_("Line number for scrolling (this line stays selected)"),config.plugins.yampmusicplayer.lyricsScrollLine))
		self.list.append(getConfigListEntry(_("Offset for time stamp in lyrics edit (milliseconds)"),config.plugins.yampmusicplayer.lyricsOffsetTime))

		self.list.append(getConfigListEntry(separator + _('        Miscellaneous         ') + separator, config.plugins.yampmusicplayer.separator))
		self.list.append(getConfigListEntry(_("Type of LCD/Graphical display used (restart required)"), config.plugins.yampmusicplayer.yampLCD))
#		self.list.append(getConfigListEntry(_("Font size for Grapical rolling text)"),config.plugins.yampmusicplayer.lcdRollingFontSize))
		self.list.append(getConfigListEntry(_("Display button description in Display"), config.plugins.yampmusicplayer.displayButtonHelp))

		self.list.append(getConfigListEntry(_("YAMP in extended plugin list"), config.plugins.yampmusicplayer.yampInExtendedPluginlist))
		self.list.append(getConfigListEntry(_("YAMP in main menu"), config.plugins.yampmusicplayer.yampInMainMenu))
		self.list.append(getConfigListEntry(_("Debug Mode"), config.plugins.yampmusicplayer.yampDebugMode))
		self.debugPath = getConfigListEntry(_("Debug directory"), config.plugins.yampmusicplayer.debugPath)
		self.list.append(self.debugPath)
		self.list.append(getConfigListEntry(_("Debug memory usage"), config.plugins.yampmusicplayer.yampDebugMemory))

		ConfigListScreen.__init__(self, self.list, session)
			
		self["title"] = Label(_("YAMP Music Player Configuration"))
		self["setupActions"] = ActionMap(["YampActions", "YampOtherActions"],
		{
			"prevBouquet": self.pagedown,
			"nextBouquet" : self.pageup,
			"red": self.keyExit,
			"green": self.keyGreen,
			"exit": self.keyExit,
			"ok": self.keyOK,
		}, -1)
		self.onLayoutFinish.append(self.layoutFinished)
		self.onClose.append(self.cleanup)
		self.triggerTimer = eTimer()
		if os.path.exists("/var/lib/dpkg/status"):
			self.triggerTimer_conn=self.triggerTimer.timeout.connect(self.triggerStartActions)
		else:
			self.triggerTimer.callback.append(self.triggerStartActions)
		self.LcdText=''
		
	def layoutFinished(self):
		self.triggerTimer.start(250,True)

	def triggerStartActions(self):
		self.updateLCDText(TEXTLCD, 1)
		
	def cleanup(self):
		del self.triggerTimer

	def updateLCDText(self, text, line):
		if config.plugins.yampmusicplayer.yampLCD.value == 'rolling':
			try:
				self.LcdText = text
#				LOG('YampConfigScreen: updateLCDText: RollingLCD: LcdText: %s' %(self.LcdText), 'err') 
			except:
				pass
		else:	
#			LOG('YampConfigScreen: updateLCDText: normalLCD: LcdText: %s' %(text), 'err') 
			self.summaries.setText(text,line)
		
	def getLcdText(self):	#for LCD Running Text
#		LOG('getLcdText: LcdText: %s' %(self.LcdText), 'all')
		return(self.LcdText)
		
	def createSummary(self):
		if config.plugins.yampmusicplayer.yampLCD.value == 'rolling':
			return YampLCDScreenRunning
		else:
			return YampLCDScreen

	def pagedown(self):
		try:
#			LOG('YampConfigScreen: pagedownspecial: Start' , 'err') 
			self["config"].instance.moveSelection(self["config"].instance.pageDown)
		except:	
			LOG('YampConfigScreen: pagedownspecial: EXCEPT' , 'err') 

	def pageup(self):
			self["config"].instance.moveSelection(self["config"].instance.pageUp)
		
	def keyOK(self):
		selection = self["config"].getCurrent()
		if selection == self.databasePath:
			self.session.openWithCallback(self.pathSelectedDatabase,YampSelectPath,config.plugins.yampmusicplayer.databasePath.value)
		elif selection == self.filebrowserPath:
			self.session.openWithCallback(self.pathSelectedFilebrowser,YampSelectPath,config.plugins.yampmusicplayer.filebrowserPath.value)
		elif selection == self.lyricsPath:
			self.session.openWithCallback(self.pathSelectedLyrics,YampSelectPath,config.plugins.yampmusicplayer.lyricsPath.value)
		elif selection == self.playlistPath:
			self.session.openWithCallback(self.pathSelectedPlaylist,YampSelectPath,config.plugins.yampmusicplayer.playlistPath.value)
		elif selection == self.screenSaverSlidePath:
			self.session.openWithCallback(self.pathSelectedScreenSaver,YampSelectPath,config.plugins.yampmusicplayer.screenSaverSlidePath.value)
		elif selection == self.screenSaverArtworkPath:
			self.session.openWithCallback(self.pathSelectedScreenSaverArtwork,YampSelectPath,config.plugins.yampmusicplayer.screenSaverArtworkPath.value)
		elif selection == self.debugPath:
			self.session.openWithCallback(self.pathSelectedDebugPath,YampSelectPath,config.plugins.yampmusicplayer.debugPath.value)

	def pathSelectedDatabase(self, result):
		if result is not None:
			config.plugins.yampmusicplayer.databasePath.value = result

	def pathSelectedFilebrowser(self, result):
		if result is not None:
			config.plugins.yampmusicplayer.filebrowserPath.value = result

	def pathSelectedLyrics(self, result):
		if result is not None:
			config.plugins.yampmusicplayer.lyricsPath.value = result

	def pathSelectedPlaylist(self, result):
		if result is not None:
			config.plugins.yampmusicplayer.playlistPath.value = result

	def pathSelectedScreenSaver(self, result):
		if result is not None:
			config.plugins.yampmusicplayer.screenSaverSlidePath.value = result

	def pathSelectedScreenSaverArtwork(self, result):
		if result is not None:
			config.plugins.yampmusicplayer.screenSaverArtworkPath.value = result

	def pathSelectedDebugPath(self, result):
		if result is not None:
			config.plugins.yampmusicplayer.debugPath.value = result

			
	def keyGreen(self):
		for x in self["config"].list:
			x[1].save()
		configfile.save()
		self.close(True)

	def keyExit(self):
		for x in self["config"].list:
			x[1].cancel()
		self.close(False)

class YampSelectPath(Screen):
	def __init__(self, session, initDir):
		Screen.__init__(self, session)
		with open(os.path.join(yampDir, "skins", config.plugins.yampmusicplayer.yampSkin.value, "YampSelectPath.xml"), 'r') as f:
			self.skin = f.read()
		inhibitDirs = ["/bin", "/boot", "/dev", "/etc", "/lib", "/proc", "/sbin", "/sys", "/usr", "/var"]
		inhibitMounts = []
		self["title"] = Label(_("YAMP Music Player - Select Path"))
		self["filelist"] = FileList(initDir, showDirectories = True, showFiles = False, inhibitMounts = inhibitMounts, inhibitDirs = inhibitDirs)
		self["target"] = Label()
		self["actions"] = ActionMap(["YampActions", "YampOtherActions"],
		{
			"exit": self.keyExit,
			"left": self.keyPageUp,
			"right": self.keyPageDown,
			"up": self.keyMoveUp,
			"down": self.keyMoveDown,
			"ok": self.keyOK,
			"green": self.keyGreen,
			"red": self.keyExit,
		}, -1)
		
	def keyExit(self):
		self.close(None)

	def keyGreen(self):
		self.close(self["filelist"].getSelection()[0])

	def keyMoveUp(self):
		self["filelist"].up()
		self.updateTarget()

	def keyMoveDown(self):
		self["filelist"].down()
		self.updateTarget()

	def keyPageUp(self):
		self["filelist"].pageUp()
		self.updateTarget()

	def keyPageDown(self):
		self["filelist"].pageDown()
		self.updateTarget()

	def keyOK(self):
		if self["filelist"].canDescent():
			self["filelist"].descent()
			self.updateTarget()

	def updateTarget(self):
		currFolder = self["filelist"].getSelection()[0]
		if currFolder is not None:
			self["target"].setText(currFolder)
		else:
			self["target"].setText(_("Invalid Location"))
		
class YampLCDScreenRunning(Screen):	  #for LCD 'Running Text'
	def __init__(self, session, parent):
		Screen.__init__(self, session, parent = parent)

		with open(os.path.join(yampDir, "skins", config.plugins.yampmusicplayer.yampSkin.value, "YampLcdRunning.xml"), 'r') as f:
			self.skin = f.read()

		self["lcdRunningText"] = StaticText("")
		self.onShow.append(self.__onShow)
		self.onHide.append(self.__onHide)
		self.lcdGetTextTimer = eTimer()
		if os.path.exists("/var/lib/dpkg/status"):
			self.lcdGetTextTimer_conn=self.lcdGetTextTimer.timeout.connect(self.selectionChanged)
		else:
			self.lcdGetTextTimer.callback.append(self.selectionChanged)



	def __onShow(self):
#		self.parent.list.connectSelChanged(self.selectionChanged)
		self.selectionChanged()

	def __onHide(self):
#		self.parent.list.disconnectSelChanged(self.selectionChanged)
		self.selectionChanged()
#		del self.lcdGetTextTimer

	def selectionChanged(self):
#		LOG('-----selectionChanged-----------', 'all')
		lcdRunningText = self.parent.getLcdText()
#		item = self.parent.getCurrentSelection()
		if lcdRunningText != self["lcdRunningText"].text:
			self["lcdRunningText"].text = lcdRunningText
		self.lcdGetTextTimer.start(1000)

	def setText(self, text, line):		#only for campatibility with standard LCD
		text = text

		
class YampLCDScreen(Screen):	#for Standard LCD 1 or 3 lines
	def __init__(self, session, parent):
		Screen.__init__(self, session)
		with open(os.path.join(yampDir, "skins", config.plugins.yampmusicplayer.yampSkin.value, "YampLCD.xml"), 'r') as f:
			self.skin = f.read()
		self["text1"] = Label("")
		self["text2"] = Label("")
		self["text3"] = Label("")

	def setText(self, text, line):
		textleer = "    "
		text = text + textleer*10
		if line == 1:
			self["text1"].setText(text)
		if config.plugins.yampmusicplayer.yampLCD.value == 'threelines':
			if line == 2:
				self["text2"].setText(text)
			elif line == 3:
				self["text3"].setText(text)

